package Aux;

use strict;

# subroutine name: max
# Input: number1, number2
# returns greater of 2 numbers
sub max {
    if ($_[0]<$_[1]) {return $_[1]} else {return $_[0]};
}

sub name_value_pairs {
  my ($line, $hash, $btw_name_value, $btw_pairs) = @_;

  #print "line $line\n";

  my @items = split (/(\w+)=/,$line);
  for (my $i = 1; $i <= $#items; $i+=2) {
    my $value = $items[$i+1];
    $value =~ s/$btw_pairs$//;
    #print "$items[$i] = $value\n";
    $hash->{$items[$i]} = $value;
  }

}

sub slurp_map {
  my ($filename, $map, $column) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    if ($line =~ /^(.*?)\s+(.*?)$/) {
      if ($column == 1) {
	$map->{$1} = $2;
      } elsif ($column == 2) {
	$map->{$2} = $1;
      } else {
	die ("Column must be 1 or 2");
      }
    } else {
      die ("Line in $filename did not match");
    }
  }
  close IN;
}

sub slurp_hash_pair {
  my ($filename, $map, $keyColumn, $valueColumn) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);
    my $key = $items[$keyColumn];
    my $value = $items[$valueColumn];
    if (defined ($key) && defined($value)) {
#	print "adding k $key v $value\n";
	$map->{$key} = $value;

    }
  }
  close IN;
}


sub slurp_column_to_hash {
  my ($filename, $hash, $column) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);
    if (defined ($items[$column])) {
      $hash->{$items[$column]} = 1;
    }
  }
  close IN;
}

sub slurp_columns_to_hash_pair {
  my ($filename, $hash, $columnA, $columnB, $num_columns) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);
    my $val = '';
    my $not_defined = 0;
    if (defined($items[$columnA])) {
      for (my $i = $columnB; $i <= $num_columns; $i++) {
	if (defined($items[$i])) {
	  $val .= $items[$i];
	  if ($i != $num_columns) {
	    $val .= ' ';
	  }
	} else {
	  $not_defined = 1;
	}
      }
      $hash->{$items[$columnA]} = $val;
    }
  }
  close IN;
}


sub slurp_column_to_hash_pair {
  my ($filename, $hash, $columnA, $columnB) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);
    if (defined ($items[$columnA]) && defined ($items[$columnB])) {
      $hash->{$items[$columnA]} = $items[$columnB];
    }
  }
  close IN;
}


sub slurp_column_to_array {
  my ($filename, $array, $column) = @_;

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);
    if (defined ($items[$column])) {
      $array->[$#$array+1] = $items[$column];
    }
  }
  close IN;
}

sub slurp_array_to_hash {
  my ($filename, $map, $keyColumn, $firstArrayElement, $distance) = @_;
  # i.e. read: key [foo1a foo1b] [foo2a foo2b]
  # return key-> array of foo1a, foo2a
  # with 0, 1, 2

  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    my (@items) = split (/\s+/,$line);

    my $key = $items[$keyColumn];
    my @elements = ();
    # read every other element, which is the feed's id
    for (my $i = $firstArrayElement; $i <= $#items; $i+=$distance) {
      $elements[$#elements+1] = $items[$i];
    }
    $map->{$key} = \@elements;
  }
  close IN;
}

sub slurp_file {
  my ($filename, $lines) = @_;
  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
      #$line =~ s/\s//g;
      chop ($line);
    $lines->[$#$lines+1] = $line;
  }
  close IN;
}

sub slurp_file_to_scalar {
  my ($filename) = @_;
  my $contents = '';
  open IN, $filename or die ("Cannot open $filename");
  while (my $line = <IN>) {
    $contents .= $line;
  }
  close IN;
  return $contents;
}

sub slurp_directory {
  my ($dirname, $files, $regex) = @_;
  opendir INDIR, $dirname or die ("Cannot open $dirname");
  my (@allfiles) = readdir (INDIR);
  closedir INDIR;
  #print "$dirname $#allfiles\n";
  foreach my $file (@allfiles) {
    #print "$file\n";
    if ($file =~ /$regex/) {
      $files->[$#$files+1] = $file;
    }
  }
}

my $ALPHA = 0.1;
sub remember_ewma {
    my ($hash, $key, $val) = @_;
    if (!defined($hash->{$key})) {
	$hash->{$key} = $val;
    } else {
	my $curr = $hash->{$key};
	$hash->{$key} = ($ALPHA*$val)+((1.-$ALPHA)*$curr);
    }
}

sub remember_all {
    my ($hash, $key, $val) = @_;
    if (!defined($hash->{$key})) {
	my @v = ();
	$hash->{$key} = \@v;
    }
    my $vals = $hash->{$key};
    $vals->[$#$vals+1] = $val;
}

sub recall_all {
    my ($hash, $key) = @_;
    if (!defined($hash->{$key})) {
	return undef;
    }
    my $vals = $hash->{$key};
    return $vals;
}

sub recall_all_to_string {
    my ($hash, $key) = @_;
    my $str = '';
    if (!defined($hash->{$key})) {
	return '';
    }
    my $vals = $hash->{$key};
    for (my $i = 0; $i <= $#$vals; $i++) {
	$str .= $vals->[$i];
	if ($i != $#$vals) {
	    $str .= ' ';
	}
    }
    return $str;
}

sub printHash {
    my ($hash) = @_;
    foreach my $key (keys %$hash) {
	print "$key->".$hash->{$key}."\n";
    }
}


1;
