#!/usr/bin/perl

use strict;
use Getopt::Std;
use Frontier::Client;
use Data::Dumper;

my $usage = "proxy.pl [options] url\n";
$usage .=   " -c remote-node -l lease: create proxy coord for remote node with lease\n";
$usage .=   " -d remote-node: destroy proxy coord for remote node\n";
$usage .=   " -r remote-node -l lease: renew lease for this proxy coord\n";
$usage .=   "\n";
$usage .=   "proxy.pl -c www.google.com -l 3600000 http://righthand.eecs.harvard.edu:55501\n";
my %para = ();
getopts('c:d:r:l:', \%para);

############################################################

if ($#ARGV < 0) {
    die ("Missing arguments\n$usage");
}
my $url = $ARGV[0];
if ($url !~ /^http:\/\/.*?:\d+$/) {
    die ("bad url: $url\n$usage\n");
}

############################################################

my $server = Frontier::Client->new(url=>"$url");

my $lease = 3600000;
if (defined($para{'l'})) {
    $lease = $para{'l'};
}

if (defined($para{'c'})) {
    my $res = $server->call("pyxida.createProxyCoord",$para{'c'},$lease);
    print Dumper($res);
    #&outputBoolean($res);
} elsif (defined($para{'d'})) {
    my $res = $server->call("pyxida.destroyProxyCoord",$para{'d'});
    print Dumper($res);
    #&outputBoolean($res);
} elsif (defined($para{'r'})) {
    my $res = $server->call("pyxida.renewProxyCoord",$para{'r'},$lease);
    print Dumper($res);
    #&outputBoolean($res);
} else {
    die ("Missing arguments\n$usage");
}

############################################################

sub outputBoolean {
    my ($res) = @_;
    #print Dumper($res);
    #&Aux::printHash($hash);
    my $success = $res->value;
    if ($success == 1) {
	print "Success\n";
    } else {
	print "Failure\n";
    }
}
